<?php
use Random\Engine;
class UserModel
{

    private EnigmeGateway $enigme_gateway;
    private PartieGateway $partie_gateway;
    private UtilisateurGateway $utilisateur_gateway;
    private ResoudreGateway $resoudre_gateway;
    private Nettoyage $nettoyage;
    private Validation $validation;

    function __construct()
    {
        try {
            global $error, $view, $rep;
            $this->enigme_gateway = new EnigmeGateway();
            $this->partie_gateway = new PartieGateway();
            $this->utilisateur_gateway = new UtilisateurGateway();
            $this->resoudre_gateway = new ResoudreGateway();
            $this->nettoyage = new Nettoyage();
            $this->validation = new Validation();
        } catch (Exception $e) {
            $error = $e->getMessage();
            require($rep . $view['erreur']);
        }
    }
    public function addToQueue($mailUtilisateur) : array{
        $idPartie=$this->partie_gateway->findPartieInQueue();
        if ($idPartie == 0){
            $enigme= $this->enigme_gateway->getRandomEnigme();
            $lesEnigmes = array($enigme);
            $this->partie_gateway->createPartieMulti($lesEnigmes, $mailUtilisateur);
            $idPartie=$this->partie_gateway->findPartieInQueue();
            $etat=$this->partie_gateway->getEtat($idPartie);
        }
        else{
            $this->partie_gateway->addToPartie($mailUtilisateur, $idPartie);
            $etat=$this->partie_gateway->getEtat($idPartie);
        }
        $tpsMaxPartie = $this->calculTpsMaXPartie($idPartie);
        $_SESSION['tpsMaxPartie'] = $tpsMaxPartie;
        return array($idPartie, $etat);
    }

    public function calculTpsMaXPartie( int $idPartie) : int {
        $lesIdEnigmes = $this->partie_gateway->findAllEnigmeIdInPartie($idPartie);
        foreach ($lesIdEnigmes as $idEnigme){
            $enigme = $this->enigme_gateway->findById($idEnigme)[0];
            $tpsMaxPartie += $enigme->getTempsDeResolution();
        }
        return $tpsMaxPartie;
    }

    public function findUserGroup() : int {
        return $this->partie_gateway->findPartieInQueue();
    }

    public function launchGame()
    {
            $this->utilisateur_gateway->launchGame($_SESSION['utilisateur']);
    }

    public function logout()
    {
        session_unset();
        session_destroy();
        $_SESSION = array();
        $_SESSION['role'] = 'visitor';
        header('Location: index.php');
    }

    public function findUsersInQueue() : array
    {
        return $this->utilisateur_gateway->findUsersInQueue();
    }

    public function getEnigmeByOrdre(int $num) : Enigme
    {
        $tabEnigme = $this->enigme_gateway->findByOrdre($num);
        if ($tabEnigme == null) {
            $last = $this->enigme_gateway->findLastEnigmaByOrdre();
            if ($last[0]->getOrdre() == ($num-1)){
                return $this->enigme_gateway->findByOrdre($num-1)[0];
            }
            throw new Exception("Enigme non trouvée");
        }
        return $tabEnigme[0];
    }

    public function resoudreEnigmeSolo(Utilisateur $utilisateur, int $enigmeId){
        if($this->resoudre_gateway->checkPartieIsAlreadyCreate($enigmeId)){
            $partieId = $this->partie_gateway->findPartieByEnigmeId($enigmeId)->getIdPartie();
        } else {
            $enigme=$this->enigme_gateway->findById($enigmeId)[0];
            $this->partie_gateway->createPartie($enigme, $utilisateur);
            $partieId = $this->partie_gateway->findLastPartie()->getIdPartie();
        }
        $this->resoudre_gateway->resoudreEnigmeSolo($utilisateur, $enigmeId, $partieId);
    }
    public function resoudreEnigmeMulti(Utilisateur $utilisateur, int $enigmeId, int $idPartie, int $index){
        $this->resoudre_gateway->resoudreEnigmeMulti($utilisateur, $enigmeId, $idPartie, $index);
    }
    public function checkEnigmeIsEnded(string $mailUtilisateur, int $enigmeId) : bool {
        return $this->resoudre_gateway->checkEnigmeIsEnded($mailUtilisateur,$enigmeId);
    }

    public function enigmeMultiEnded(string $mailUtilisateur, int $enigmeId){
        $idPartie = $_SESSION['idPartie'];
        $dateDebut = $this->partie_gateway->getDateDebut($idPartie);
        $now = new DateTime();
        $interval = $now->diff($dateDebut);
        $tempDeResolution = $interval->days * 24 * 60 * 60 + $interval->h * 60 * 60 + $interval->i * 60 + $interval->s;
        $result = $this->resoudre_gateway->getMaxClassement($enigmeId, $idPartie);
        if ($result == 0){
            $classement = 1;
        }
        else{
            $classement = $result + 1;
        }
        $this->resoudre_gateway->enigmeMultiEnded($mailUtilisateur,$enigmeId, $tempDeResolution, $classement);
    }

    public function saveCode(string $mailUtilisateur, int $enigmeId,string $code ){
        $this->resoudre_gateway->saveCode($mailUtilisateur, $enigmeId, $code);
    }
    public function saveCodeMulti(string $mailUtilisateur, int $enigmeId, int $idPartie, int $index,string $code ){
        $this->resoudre_gateway->saveCodeMulti($mailUtilisateur, $enigmeId, $idPartie , $index, $code);
    }

    public function getCode(string $mailUtilisateur, int $enigmeId){
        return $this->resoudre_gateway->getCode($mailUtilisateur, $enigmeId);
    }

    public function getLastEnigmeEnded(string $mailUtilisateur) : Enigme{
        $id=$this->resoudre_gateway->getLastEnigmeEnded($mailUtilisateur);
        if ($id == 0){
            return $this->enigme_gateway->findByOrdre(1)[0];
        }
        return $this->enigme_gateway->findById($id)[0];
    }

    public function getEtatPartie($idPartie){
        return $this->partie_gateway->getEtat($idPartie);
    }

    public function getLesJoueurs($idPartie){
        $lesMailJoeurs = $this->partie_gateway->getLesMailJoueurs($idPartie);
        $lesJoueurs = array();
        foreach ($lesMailJoeurs as $mail){
            $lesJoueurs[] = $this->utilisateur_gateway->getUtilisateurByEmail($mail);
        }
        return $lesJoueurs;
    }

    public function getEnigmebyPartieIdAndIndex($idPartie, $index) : Enigme{
        $idEnigme = $this->partie_gateway->findEnigmeIdInPartieWithIndex($idPartie, $index);
        return $this->enigme_gateway->findById($idEnigme)[0];
    }

    public function findEnigmeById(int $enigmeId) : Engine{
        return $this->enigme_gateway->findById($enigmeId)[0];
    }

    public function getLastIndex(int $idPartie) : int{
        return $this->resoudre_gateway->getLastIndex($idPartie);
    }

    public function getLastOrdre() :int {
        return $this->enigme_gateway->getLastOrdre();
    }

    public function getDateDebut($idPartie) : DateTime{
        return $this->partie_gateway->getDateDebut($idPartie);
    }

    public function getPointsAtTheEnd(string $mailUtilisateur, int $idPartie) : int{
        $points = 0;
        $result = $this->resoudre_gateway->getAllByPartieAndUtilisateur($mailUtilisateur, $idPartie);
        foreach ($result as $row){
            if ($row['classement'] == 1){
                $points += $this->enigme_gateway->findById($row['enigme'])[0]->getPoints();
            }
            else if ($row['classement'] == 2){
                $points += $this->enigme_gateway->findById($row['enigme'])[0]->getPoints() * 0.75;
            }
            else{
                $points += $this->enigme_gateway->findById($row['enigme'])[0]->getPoints() * 0.5;
            }
        }
        return $points;
    }
}